import datetime

from django.contrib.auth.mixins import LoginRequiredMixin
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.views import generic

from gym.mixins import SessionResetMixin, QuickActionsMixin
from gym.models import Room, Equipment, Setting, Session, Round, TheoreticalMax


class RoomListView(LoginRequiredMixin, SessionResetMixin, generic.ListView):
    queryset = Room.objects.all().order_by('name')
    context_object_name = 'rooms'
    template_name = 'gym/rooms.html'


class RoomDetailView(LoginRequiredMixin, QuickActionsMixin, SessionResetMixin, generic.DetailView):
    model = Room
    context_object_name = 'room'
    template_name = 'gym/room.html'

    def get_quick_actions(self):
        return [
            {
                'url': reverse('rooms-list'),
                'category': 'secondary',
                'display': 'Liste des salles'
            },
            {
                'url': "{}?room={}".format(reverse('equipment-create'), self.object.pk),
                'category': 'success',
                'display': 'Ajouter une machine'
            },
            {
                'url': "{}?room={}".format(reverse('session-start'), self.object.pk),
                'category': 'primary',
                'display': 'Commencer une séance'
            },
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['sessions'] = self.object.sessions.all().order_by('-start')
        return context


class EquipmentDetailView(LoginRequiredMixin, generic.DetailView):
    model = Equipment
    context_object_name = 'equipment'
    template_name = 'gym/equipment.html'
    session = None

    def dispatch(self, request, *args, **kwargs):
        session_pk = self.request.session.get('session_pk')
        if session_pk:
            self.session = Session.objects.get(pk=session_pk)
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.session:
            context['session'] = self.session.pk
            context['rounds'] = Round.objects.filter(session=self.session, equipment=self.get_object())
        return context


class EquipmentCreateView(LoginRequiredMixin, QuickActionsMixin, generic.CreateView):
    model = Equipment
    fields = ['room', 'name']
    template_name = 'gym/equipment_edit.html'
    room = None

    def get_quick_actions(self):
        return [
            {
                'url': reverse('room-detail', args=(self.room.pk,)),
                'category': 'secondary',
                'display': 'Retourner à la salle'
            },
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['room'] = self.room
        return context

    def get_success_url(self):
        return reverse('equipment-detail', kwargs={'pk': self.object.pk})

    def dispatch(self, request, *args, **kwargs):
        default_room_id = self.request.GET.get('room')
        if default_room_id:
            self.room = get_object_or_404(Room, pk=default_room_id)

        return super().dispatch(request, *args, **kwargs)

    def get_initial(self):
        initial = super().get_initial()
        initial['room'] = self.room

        return initial


class SettingCreateView(LoginRequiredMixin, QuickActionsMixin, generic.CreateView):
    model = Setting
    fields = ['equipment', 'name', 'value']
    template_name = 'gym/setting_edit.html'
    equipment = None

    def get_quick_actions(self):
        return [
            {
                'url': reverse('equipment-detail', args=(self.equipment.pk,)),
                'category': 'secondary',
                'display': 'Retourner à la machine'
            },
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = 'Ajouter'
        context['equipment'] = self.equipment
        return context

    def dispatch(self, request, *args, **kwargs):
        default_equipment_id = self.request.GET.get('equipment')
        if default_equipment_id:
            self.equipment = get_object_or_404(Equipment, pk=default_equipment_id)

        return super().dispatch(request, *args, **kwargs)

    def get_initial(self):
        initial = super().get_initial()
        initial['equipment'] = self.equipment

        return initial

    def get_success_url(self):
        return reverse('equipment-detail', kwargs={'pk': self.object.equipment.pk})


class SettingUpdateView(LoginRequiredMixin, QuickActionsMixin, generic.UpdateView):
    model = Setting
    fields = ['equipment', 'name', 'value']
    template_name = 'gym/setting_edit.html'

    def get_quick_actions(self):
        return [
            {
                'url': reverse('equipment-detail', args=(self.object.equipment.pk,)),
                'category': 'secondary',
                'display': 'Retourner à la machine'
            },
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['edit'] = True
        context['title'] = 'Modifier'
        context['equipment'] = self.object.equipment
        return context

    def get_success_url(self):
        return reverse('equipment-detail', kwargs={'pk': self.object.equipment.pk})


class SettingDeleteView(LoginRequiredMixin, generic.DeleteView):
    model = Setting
    template_name = 'gym/setting_confirm_delete.html'
    context_object_name = 'setting'

    def get_success_url(self):
        return reverse('equipment-detail', kwargs={'pk': self.object.equipment.pk})


class TheoreticalMaxCreateView(LoginRequiredMixin, QuickActionsMixin, generic.CreateView):
    model = TheoreticalMax
    fields = ['equipment', 'date', 'value']
    template_name = 'gym/theoretical_max_edit.html'
    equipment = None

    def get_quick_actions(self):
        return [
            {
                'url': reverse('equipment-detail', args=(self.equipment.pk,)),
                'category': 'secondary',
                'display': 'Retourner à la machine'
            },
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = 'Ajouter'
        context['equipment'] = self.equipment
        return context

    def dispatch(self, request, *args, **kwargs):
        default_equipment_id = self.request.GET.get('equipment')
        if default_equipment_id:
            self.equipment = get_object_or_404(Equipment, pk=default_equipment_id)

        return super().dispatch(request, *args, **kwargs)

    def get_initial(self):
        initial = super().get_initial()
        initial['equipment'] = self.equipment
        initial['date'] = datetime.date.today()

        return initial

    def get_success_url(self):
        return reverse('equipment-detail', kwargs={'pk': self.object.equipment.pk})


class SessionCreateView(LoginRequiredMixin, QuickActionsMixin, generic.CreateView):
    model = Session
    fields = ['room', 'start']
    template_name = 'gym/session_edit.html'
    room = None

    def get_quick_actions(self):
        room_pk = self.room.pk
        return [
            {
                'url': reverse('room-detail', args=(room_pk,)),
                'category': 'secondary',
                'display': 'Retourner à la salle'
            },
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = 'Démarrer'
        context['room'] = self.room
        return context

    def get_success_url(self):
        return reverse('session-detail', kwargs={'pk': self.object.pk})

    def dispatch(self, request, *args, **kwargs):
        self.room = get_object_or_404(Room, pk=self.request.GET.get('room'))
        return super().dispatch(request, *args, **kwargs)

    def get_initial(self):
        initial = super().get_initial()
        initial['room'] = self.room
        initial['start'] = datetime.datetime.now()

        return initial


class SessionDetailView(LoginRequiredMixin, QuickActionsMixin, generic.DetailView):
    model = Session
    context_object_name = 'session'
    template_name = 'gym/session_detail.html'

    def get_quick_actions(self):
        room_pk = self.get_object().room.pk
        return [
            {
                'url': reverse('room-detail', args=(room_pk,)),
                'category': 'secondary',
                'display': 'Retourner à la salle'
            },
            {
                'url': "{}?room={}".format(reverse('equipment-create'), room_pk),
                'category': 'success',
                'display': 'Ajouter une machine'
            },
        ]

    def dispatch(self, request, *args, **kwargs):
        request.session['session_pk'] = self.get_object().pk
        return super().dispatch(request, *args, **kwargs)


class RoundCreateView(LoginRequiredMixin, QuickActionsMixin, generic.CreateView):
    model = Round
    fields = ['equipment', 'session', 'theoretical_max_percentage', 'chosen_weight', 'repetition_number', 'notes']
    template_name = 'gym/round_edit.html'
    equipment = None
    session_pk = None

    def get_quick_actions(self):
        return [
            {
                'url': reverse('equipment-detail', args=(self.equipment.pk,)),
                'category': 'secondary',
                'display': 'Retourner à la machine'
            },
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = 'Commencer'
        context['equipment'] = self.equipment
        return context

    def dispatch(self, request, *args, **kwargs):
        self.equipment = get_object_or_404(Equipment, pk=self.request.GET.get('equipment'))
        self.session_pk = self.request.session.get('session_pk')

        return super().dispatch(request, *args, **kwargs)

    def get_initial(self):
        theoretical_max_percentage = 65
        initial = super().get_initial()
        initial['equipment'] = self.equipment
        initial['session'] = self.session_pk
        initial['repetition_number'] = 12
        initial['theoretical_max_percentage'] = theoretical_max_percentage
        proposed_weight = 0
        theoretical_max = self.equipment.last_theoretical_max
        if theoretical_max:
            proposed_weight = theoretical_max.value * theoretical_max_percentage / 100
        initial['chosen_weight'] = proposed_weight

        return initial

    def get_success_url(self):
        return reverse('equipment-detail', kwargs={'pk': self.object.equipment.pk})
